import React, { Fragment , useEffect , useState } from 'react';
import moment from "moment";
import { List , Button , Empty } from 'antd';
import { isObject} from "lodash";
import { GetDeviceRealInfo, SensorsDataForStation} from "../../../../apis/process";
import { ReloadOutlined } from "@ant-design/icons"
const IndicatorCard = props => {
  const { schema , addons } = props;
  const [list , setList] = useState([]);
  const [loading , setLoading] = useState(false);

  const loopObj = (obj , newObj) => {
    for (let key in obj) {
      if(isObject(obj[key])){
        loopObj(obj[key] , newObj)
      } else {
        newObj[key] = obj[key];
      }
    }
  }

  const getMappedField = () => {
    let field = "";

    if(addons){
      const newObj = {};
      const addonsForm = addons.getValues();
      loopObj(addonsForm , newObj);
      field = newObj[schema.fieldName]
    }

    return field
  }

  const getDeviceData = async () => {
    const { indicatorType , timeInterval } = schema;
    const arr = [];

    setLoading(true);

    if(schema.dataType === "real"){
      const ret = await GetDeviceRealInfo({
        userID: 0,
        pageIndex: 1,
        pageSize: 100,
        param: [{
          deviceType:  schema.deviceType,
          deviceCode: schema.deviceCode,
        }],
      });

      if(ret.code === 0){
        ret.data.list.forEach((item) => {
          item.dataList.forEach((child) => {
            if(indicatorType.indexOf(child.dName) !== -1){
              arr.push({
                name: child.dName,
                value: child.pv,
                time: item.pt,
                unit: schema._IndicatorType[child.dName]
              })
            }
          })
        })
      }

    } else {
      const currentField = getMappedField();

      if(currentField){
        const timeArr = timeInterval.split(",");
        const interval = timeArr[0];
        const unit = timeArr[1];
        const date = moment(currentField);

        const dateFrom = date.add(-interval , unit).format("YYYY-MM-DD HH:mm:ss");
        const dateTo = date.add(interval, unit).format("YYYY-MM-DD HH:mm:ss");
        const acrossTables = [];

        const indicatorTypeList = indicatorType.split(',');

        indicatorTypeList.forEach(item => {
          acrossTables.push({
            deviceCode: schema.deviceCode,
            sensors: item,
            deviceType: schema.deviceType,
          })
        })

        const obj = {
          ignoreOutliers: false,
          isVertical: false,
          acrossTables,
          dateFrom,
          dateTo,
        }
        const ret = await SensorsDataForStation(obj);

        if(ret.code === 0){
          ret.data.forEach((item) => {
            const dataList = item.dataModel;
            const currentData = dataList[dataList.length - 1];

            arr.push({
              name: item.sensorName,
              value: currentData?.pv || "未获取到数据",
              time: currentData?.pt || "未获取到数据",
              unit: item.unit
            })
          })
        }
      }
    }


    setList(arr);
    setLoading(false);
  };

  const refreshData = () => {
    getDeviceData();
  }

  useEffect(() => {
    if(schema.deviceType &&
      schema.dataType &&
      schema.indicatorType &&
      schema.deviceCode){
      getDeviceData()
    } else {
      setList([])
    }
  } , [schema])

  return (
    <Fragment>
      <List>
        {
          list.length > 0 ? list.map((item , index) => {
            return (
              <List.Item key={index}
                         actions={[<span>{item.time}</span>]}
                         extra={<Button onClick={() => refreshData(item)}
                                        size="small"
                                        shape="circle"
                                        loading={loading}
                                        icon={<ReloadOutlined spin={loading} />} />}>
                <List.Item.Meta title={item.name}
                                description={`${item.value || 0}  ${item.unit || ""}`} />
              </List.Item>
            )
          }) : (
            <List.Item>
              <Empty />
            </List.Item>
          )
        }
      </List>
    </Fragment>
  );
};

export default IndicatorCard;
