import React, { Fragment, useEffect, useState } from 'react';
import { Button, Empty, Input, Space, Table } from 'antd';
import * as AntdIcon from '@ant-design/icons';
import FileView from '../../../Account/components/TablePack/components/FileView';
import TagPack from '../../../components/TagPack';
import eventEmitter from '../../../../utils/eventEmitter';

import { isArray } from 'lodash';

import styles from './index.less';

const DataTable = props => {
  const { value, onChange, schema, addons } = props;
  const {
    disabled,
    fieldList,
    optionRender,
    titleShow,
    pagination,
    scrollX,
    scrollY,
    statusOption,
    areaTaskShine,
    optionWidth,
    optionFixed,
    isOpenRowEvent
  } = schema;

  const [dataSource, setDataSource] = useState([]);
  const [visible, setVisible] = useState(true);
  const [columns, setColumns] = useState([]);

  const doAction = (_, row, index, actionItem) => {
    const list = [...dataSource];

    switch (actionItem.action) {
      case 'delete':
        list.splice(index, 1);
        setDataSource(list);
        break;
      case 'update':
        list.forEach((item, i) => {
          if (i === index) {
            if (!row.isEditable) {
              row.isEditable = true;
              row.type = row.type || 'update';
            } else {
              delete row.isEditable;
            }
          }
        });
        setDataSource(list);
        break;
      default:
        break;
    }

    eventEmitter.emit(`${addons?.dataPath}_event`, {
      ...row,
      action: actionItem.action,
    });
  };

  const changeValue = (val, index, type) => {
    const list = [...dataSource];

    list.forEach((item, i) => {
      if (i === index) {
        item[type] = val;
      }
    });

    setDataSource(list);
  };

  const optionItemRender = ({ item, _, row, index, i, type }) => {
    let Element;

    if (!item.isMapped) {
      let otherProps;
      if (type === 'icon') {
        const Icon = AntdIcon[item.value];

        if(Icon){
          otherProps = {
            type: 'text',
            shape: 'circle',
            icon: <Icon />,
          };
        }

      }

      Element = (
        <Button size={'small'} key={i} onClick={() => doAction(_, row, index, item)} type={type} {...otherProps}>
          {type !== 'icon' && item.value}
        </Button>
      );
    } else {
      if (item.parseStr) {
        const json = JSON.parse(item.parseStr);
        const field = _[item.value];

        const optionCode = json[field]?.code;

        let otherProps;
        if (type === 'icon') {
          const Icon = AntdIcon[json[field]?.label];
          if(Icon){
            otherProps = {
              type: 'text',
              shape: 'circle',
              icon: <Icon style={{color:json[_[item.value]]?.color}} />,
            };
          }
        }


        if (optionCode) {
          if (field) {
            const codeStr = `return row[item.value] === "${field}"${optionCode}`;
            const deFun = new Function('_', 'row', 'index', 'item', codeStr);
            const isVisible = deFun(_, row, index, item);

            if (isVisible) {
              Element = (
                <Button
                  size={'small'}
                  key={i}
                  onClick={() => doAction(_, row, index, item)}
                  type={type}
                  {...otherProps}
                >
                  {type !== 'icon' && <span style={{ color: json[field]?.color }}>{json[field]?.label}</span>}
                </Button>
              );
            }
          }
        } else {
          if (field) {
            Element = (
              <Button size={'small'} key={i} onClick={() => doAction(_, row, index, item)} type={type} {...otherProps}>
                {type !== 'icon' && <span style={{ color: json[field]?.color }}>{json[field]?.label}</span>}
              </Button>
            );
          }
        }
      } else {
        Element = '调试中...';
      }
    }

    return Element;
  };

  // 操作栏渲染
  const getOptionRender = (_, row, index) => {
    const eleList = [];

    optionRender?.forEach((item, i) => {
      const Element = optionItemRender({
        item,
        _,
        row,
        index,
        i,
        type: item.type,
      });

      eleList.push(Element);
    });

    return (
      <Fragment>
        <Space>{eleList}</Space>
      </Fragment>
    );
  };

  // 字段栏显示模式渲染
  const getFieldRender = (_, row, index, item) => {
    let ele;

    switch (item.type) {
      case '文本':
        if (!item.isMapped) {
          ele = <span style={{ color: item.color }}>{_}</span>;
        } else {
          if (item.parseStr) {
            const json = JSON.parse(item.parseStr);
            ele = <span style={{ color: json[_]?.color }}>{json[_]?.label}</span>;
          } else {
            ele = <span>调试中...</span>;
          }
        }
        break;
      case 'tag':
        if (!item.isMapped) {
          ele = <TagPack color={item.color} text={_} />;
        } else {
          if (item.parseStr) {
            const json = JSON.parse(item.parseStr);
            ele = <TagPack color={json[_]?.color} text={json[_]?.label} />;
          } else {
            ele = <span>调试中...</span>;
          }
        }
        break;
      case 'FileUpload':
        ele = <FileView value={_} download={true} preview={true} />;
        break;
      default:
        if (!item.isMapped) {
          ele = <span style={{ color: item.color }}>{_}</span>;
        } else {
          if (item.parseStr) {
            const json = JSON.parse(item.parseStr);
            ele = <span style={{ color: json[_]?.color }}>{json[_]?.label}</span>;
          } else {
            ele = <span>调试中...</span>;
          }
        }
        break;
    }

    return ele;
  };

  const getColumns = () => {
    const arr = [];
    const list = statusOption;

    let isGIS = false;
    list?.forEach(item => {
      if (item.fieldType === 'gis') {
        isGIS = true;
      }
    });

    list?.forEach(item => {
      const dataIndex = !isGIS ? item.fieldName : `${item.fieldType}-${item.fieldName}`;

      arr.push({
        title: item.fieldAliasName,
        dataIndex,
        key: dataIndex,
        width: item.width || 120,
        fixed: item.fixed || false,
        render: (_, row, index) => {
          return (
            <Fragment>
              {row.isEditable ? (
                <Input value={_} onChange={e => changeValue(e.target.value, index, dataIndex)} />
              ) : (
                getFieldRender(_, row, index, item)
              )}
            </Fragment>
          );
        },
      });
    });

    if (!disabled) {
      arr.push({
        title: '操作',
        dataIndex: '',
        key: '',
        width: optionWidth || 120,
        fixed: optionFixed,
        render: (_, row, index) => {
          return getOptionRender(_, row, index);
        },
      });
    }

    setColumns(arr);
  };

  const initRelationForm = () => {
    window.relationForm = {
      configs: [
        {
          台账名称: schema.accountName,
          映射字段: [{ fromField: '工单编号', toField: '工单编号' }],
          widget: schema.widget,
          layerName: schema.layerName,
          accountName: schema.accountName,
          tableName: schema.tableName,
        },
      ],
      data: [],
    };
  };

  const setRelationData = () => {
    const data = [];

    dataSource.forEach(item => {
      const obj = {
        type: item.type || 'add',
        accountTable: schema.accountName,
        values: [
          {
            fieldName: '工单编号',
            fieldValue: '',
          },
        ],
      };

      Object.keys(item).forEach(key => {
        const val = key.split('-')[1];
        const isHas = areaTaskShine.find(s => s.fromField === val);

        if (isHas && item[key]) {
          const valuesObj = {
            fieldName: isHas.fromField,
            fieldValue: item[key],
          };

          obj.values.push(valuesObj);
        }
      });

      data.push(obj);
    });

    window.relationForm.data = data;
  };

  const rowClickEvent = (record) => {
    return {
      onClick: () => {
        eventEmitter.emit(`${addons?.dataPath}_event`, {
          ...record,
          action: "rowOnClick",
        });
      }
    }
  }

  useEffect(() => {
    eventEmitter.removeAllListeners(addons?.dataPath);

    let list = [...dataSource];

    eventEmitter.on(addons?.dataPath, res => {
      if (isArray(res)) {
        list = [].concat(res);
      } else {
        if (list.some(v => v['gis-GIS编号'] === res['gis-GIS编号'])) return;
        list.push(res);
      }

      setVisible(false);

      list.forEach((item, index) => {
        item.key = index;
      });
      setDataSource(list);
      setVisible(true);
      onChange(list.length);
    });

    return () => {
      eventEmitter.removeAllListeners(addons?.dataPath);
    };
  }, [dataSource]);

  useEffect(() => {
    getColumns();
  }, [fieldList, dataSource , optionRender]);

  useEffect(() => {
    initRelationForm();

    setRelationData();
  }, [value, dataSource]);

  return (
    <Fragment>
      <div className={styles.dataTable}>
        <div className={styles.tableTitle}>{titleShow}</div>
        {visible && columns.length !== 0 ? (
          <Table
            size="small"
            rowKey={disabled ? 'gis-设备编号' : 'ID'}
            bordered
            columns={columns}
            pagination={pagination}
            scroll={
              !pagination && {
                x: scrollX,
                y: scrollY,
              }
            }
            onRow={
              record => isOpenRowEvent && rowClickEvent(record)
            }
            dataSource={dataSource}
          />
        ) : (
          <Empty />
        )}
      </div>
    </Fragment>
  );
};

export default DataTable;
